import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useMemo, Fragment, useState } from "react";
import { CollapseIcon } from "../../icons/CollapseIcon";
/**
 *
 * Format component stack into pseudo HTML
 * component stack is an array of strings, e.g.: ['p', 'p', 'Page', ...]
 *
 * For html tags mismatch, it will render it for the code block
 *
 * ```
 * <pre>
 *  <code>{`
 *    <Page>
 *       <p>
 *       ^^^
 *         <p>
 *         ^^^
 *  `}</code>
 * </pre>
 * ```
 *
 * For text mismatch, it will render it for the code block
 *
 * ```diff
 * <pre>
 * <code>{`
 *   <Page>
 *     <p>
 * -     "Server Text"
 * +     "Client Text"
 *     </p>
 *   </Page>
 * `}</code>
 * ```
 */ export function PseudoHtmlDiff(param) {
    let { componentStackFrames, serverContent, clientContent, hydrationMismatchType, ...props } = param;
    const isHtmlTagsWarning = hydrationMismatchType === "tag";
    // For text mismatch, mismatched text will take 2 rows, so we display 4 rows of component stack
    const MAX_NON_COLLAPSED_FRAMES = isHtmlTagsWarning ? 6 : 4;
    const shouldCollapse = componentStackFrames.length > MAX_NON_COLLAPSED_FRAMES;
    const [isHtmlCollapsed, toggleCollapseHtml] = useState(shouldCollapse);
    const htmlComponents = useMemo(()=>{
        const tagNames = isHtmlTagsWarning ? [
            serverContent,
            clientContent
        ] : [];
        const nestedHtmlStack = [];
        let lastText = "";
        componentStackFrames.map((frame)=>frame.component).reverse().forEach((component, index, componentList)=>{
            const spaces = " ".repeat(nestedHtmlStack.length * 2);
            const prevComponent = componentList[index - 1];
            const nextComponent = componentList[index + 1];
            // When component is the server or client tag name, highlight it
            const isHighlightedTag = tagNames.includes(component);
            const isRelatedTag = isHighlightedTag || tagNames.includes(prevComponent) || tagNames.includes(nextComponent);
            const isLastFewFrames = !isHtmlTagsWarning && index >= componentList.length - 6;
            if (isHtmlTagsWarning && isRelatedTag || isLastFewFrames) {
                const codeLine = /*#__PURE__*/ _jsxs("span", {
                    children: [
                        spaces,
                        /*#__PURE__*/ _jsx("span", {
                            ...isHighlightedTag ? {
                                "data-nextjs-container-errors-pseudo-html--tag-error": true
                            } : undefined,
                            children: "<" + component + ">\n"
                        })
                    ]
                });
                lastText = component;
                const wrappedCodeLine = /*#__PURE__*/ _jsxs(Fragment, {
                    children: [
                        codeLine,
                        isHighlightedTag && /*#__PURE__*/ _jsx("span", {
                            children: spaces + "^".repeat(component.length + 2) + "\n"
                        })
                    ]
                }, nestedHtmlStack.length);
                nestedHtmlStack.push(wrappedCodeLine);
            } else {
                if (nestedHtmlStack.length >= MAX_NON_COLLAPSED_FRAMES && isHtmlCollapsed) {
                    return;
                }
                if (!isHtmlCollapsed || isLastFewFrames) {
                    nestedHtmlStack.push(/*#__PURE__*/ _jsxs("span", {
                        children: [
                            spaces,
                            "<" + component + ">\n"
                        ]
                    }, nestedHtmlStack.length));
                } else if (isHtmlCollapsed && lastText !== "...") {
                    lastText = "...";
                    nestedHtmlStack.push(/*#__PURE__*/ _jsxs("span", {
                        children: [
                            spaces,
                            "...\n"
                        ]
                    }, nestedHtmlStack.length));
                }
            }
        });
        if (hydrationMismatchType === "text") {
            const spaces = " ".repeat(nestedHtmlStack.length * 2);
            const wrappedCodeLine = /*#__PURE__*/ _jsxs(Fragment, {
                children: [
                    /*#__PURE__*/ _jsx("span", {
                        "data-nextjs-container-errors-pseudo-html--diff-remove": true,
                        children: spaces + ('"' + serverContent + '"\n')
                    }),
                    /*#__PURE__*/ _jsx("span", {
                        "data-nextjs-container-errors-pseudo-html--diff-add": true,
                        children: spaces + ('"' + clientContent + '"\n')
                    })
                ]
            }, nestedHtmlStack.length);
            nestedHtmlStack.push(wrappedCodeLine);
        }
        return nestedHtmlStack;
    }, [
        componentStackFrames,
        isHtmlCollapsed,
        clientContent,
        serverContent,
        isHtmlTagsWarning,
        hydrationMismatchType,
        MAX_NON_COLLAPSED_FRAMES
    ]);
    return /*#__PURE__*/ _jsxs("div", {
        "data-nextjs-container-errors-pseudo-html": true,
        children: [
            /*#__PURE__*/ _jsx("button", {
                tabIndex: 10,
                "data-nextjs-container-errors-pseudo-html-collapse": true,
                onClick: ()=>toggleCollapseHtml(!isHtmlCollapsed),
                children: /*#__PURE__*/ _jsx(CollapseIcon, {
                    collapsed: isHtmlCollapsed
                })
            }),
            /*#__PURE__*/ _jsx("pre", {
                ...props,
                children: /*#__PURE__*/ _jsx("code", {
                    children: htmlComponents
                })
            })
        ]
    });
}

//# sourceMappingURL=component-stack-pseudo-html.js.map